// ---------------------------------------------------------------------
/**
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

 \file MICARY_struct.h
 \brief \~English Header file for Microphone Array System
 \brief \~Japanese }CNACVXe̋ʍ\̂̂߂̃wb_t@C(Microphone Array System)
 \author Natsu Tanaka 
 \date 20 Jul 2005
 \version $Id: MICARY_struct.h 3538 2012-02-29 07:25:35Z tanaka $
*/

// ---------------------------------------------------------------------
#ifndef	MICARY_struct_h_defined
#define	MICARY_struct_h_defined
// ---------------------------------------------------------------------
#include "MICARY_def.h"	// 2011/11/16 added by N.Tanaka
#include "MICARYHA_struct.h"
// modified by S.Suzuki 2010/03/28
#include <stdio.h>
//#include "wave.h"	// 2006/09/29 added by N.Tanaka
//#include "SpectrumFile.h"	// 2006/09/29 added by N.Tanaka	// 2008/11/11 deleted by N.Tanaka

// 2008/11/11 added by N.Tanaka
// micarray_fdpluginp̑΍
#ifndef HA_STRING_MAXLEN
#define HA_STRING_MAXLEN     1024
#endif
// ---------------------------------------------------------------------

/**
 TF Calclating Flag Enum
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef enum{
	ntcFreeField,
	ntcReflection1p,
	ntcReflection1m,
} MICARY_TFCalcEnum;

/**
 Frequency Scale Enum
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef enum{
	nfsLog,
	nfsLinear
} MICARY_FreqScaleEnum;

/**
 Kind of window Enum
 \author	Natsu Tanaka
 \date	19 Oct 2005
*/
typedef enum{
	nwnHanning,
	nwnSpecial
} MICARY_WindowEnum;

/**
 Location Flag
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef int	MICARY_LocationFlg;

/**
 Float array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Float *	ha_fValue; ///<\~Japanese HA_Floatz
	HA_Num	ha_numValue; ///<\~Japanese z̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_FloatArray;

/**
 Integer array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Integer *	ha_iValue; ///<\~Japanese HA_Integerz
	HA_Num	ha_numValue; ///<\~Japanese z̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_IntArray;

/**
 Complex array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Complex *	ha_cmpValue; ///<\~Japanese HA_Complexz
	HA_Num	ha_numValue; ///<\~Japanese z̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_ComplexArray;

/**
 Location struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Float	ha_fX; ///<\~Japanese XW[m]
	HA_Float	ha_fY; ///<\~Japanese YW[m]
	HA_Float	ha_fZ; ///<\~Japanese ZW[m]
	HA_Float	ha_fTheta; ///<\~Japanese ʊp[deg]
	HA_Float	ha_fPhi; ///<\~Japanese p[deg]
	MICARY_LocationFlg	micary_LocationFlag; ///<\~Japanese LȍWtO
	HA_Id	ha_idPos; ///<\~Japanese ʒuID // 2008/05/15 added by N.Tanaka
	HA_Id ha_idDir; ///<\~Japanese ID // 2008/05/15 added by N.Tanaka
	HA_Id tfindex[3]; ///<\~Japanese TFIndex // 2013/06/26 added by K.Morio
	HA_Id tfindex_max[3]; ///<\~Japanese TFIndex̍ől // 2013/10/10 added by K.Morio
    HA_Id tfindex_min[3]; ///<\~Japanese TFIndex̍ŏl // 2013/10/10 added by K.Morio
	HA_Id tfposid;	///<\~Japanese TFDB̈ʒuID // 2014/8/27 added by N.Tanaka ha_idPosقȂgĂ邽ߒǉ

} MICARY_Location;

/**
 Location Array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Id *	ha_idLocation; ///<\~Japanese ʒuWIDz
	MICARY_Location *	micary_Location; ///<\~Japanese ʒuWz
	HA_Num	ha_numLocation; ///<\~Japanese ʒuW
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_LocationArray;

/**
 Parameter of setting struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	HA_Float	ha_fFs; ///<\~Japanese TvOg[Hz]
	HA_Num		ha_numNFFT; ///<\~Japanese FFT
	HA_Num		ha_numWindowLength; ///<\~Japanese 
	MICARY_FreqScaleEnum	micary_FScale; ///<\~Japanese g
	HA_Float	ha_fTemperature; ///<\~Japanese C[deg C]
} MICARY_SettingParameter;

/**
 Database of BF
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	MICARY_ComplexArray	micary_cmpaBF; ///<\~Japanese BFWz
	MICARY_IntArray		micary_iaFreqLine; ///<\~Japanese BFW̎gCz
	MICARY_LocationArray	micary_locaSource; ///<\~Japanese BFW̉ʒuWz
	MICARY_LocationArray	micary_locaMic; ///<\~Japanese BFW̃}CNzʒuWz
	HA_Float		ha_fSearchMicR; ///<\~Japanese BFvZΏۂƂȂ}CNzƉTʒű̏[m]
	//HA_Num			ha_numMic;
} MICARY_BFDatabase;

/**
 Database of TF
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	MICARY_ComplexArray	micary_cmpaTF; ///<\~Japanese `B֐z
	MICARY_IntArray	micary_iaFreqLine; ///<\~Japanese `B֐̎gCz
	MICARY_LocationArray	micary_locaSource; ///<\~Japanese `B֐̉ʒuWz
	MICARY_LocationArray	micary_locaMic; ///<\~Japanese `B֐̃}CNzʒuWz
	//HA_Num			ha_numMic;
} MICARY_TFDatabase;

/**
 Integer array struct(dim:2)
 \author	Natsu Tanaka
 \date	12 Oct 2005
*/
typedef struct{
	HA_Integer **	ha_iValue; ///<\~Japanese HA_Integer^2z
	HA_Num	ha_numValue1; ///<\~Japanese z̃TCY
	HA_Num	ha_numValue2; ///<\~Japanese z̃TCY
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReserved1; ///<\~Japanese z̎ۂ̃TCY
	HA_Num	ha_numReserved2; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_IntArray2;

/**
 ADC data array struct
 \author	Natsu Tanaka
 \date 5 Dec 2005
*/
typedef struct{
	HA_Num	ha_numDataLength; ///<\~Japanese LȒl̐
	HA_Num	ha_numCh;	///<\~Japanese AD`l // 2006/11/09 added by N.Tanaka
	MICARY_IntArray micary_iaData; ///<\~Japanese mۍς݂HA_Integer̔z
} MICARY_ADCDataArray;

// modified S.Suzuki 2010/03/28
/**
 Wavfile information struct
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
/*
typedef struct{
	//char	cFilename[256]; ///<\~Japanese t@C
	HA_String ha_stFileName; ///<\~Japanese t@C
	FILE *	pfinFile; ///<\~Japanese t@C|C^
	HA_Num	ha_numCurrentSample; ///<\~Japanese ǂݏoς݃Tv[sample]
	struct waveheader sWavHeader; ///<\~Japanese wavt@Cwb_
} MICARY_WavFile;
*/
// 2008/11/11 moved from SpectrumFile.h
// 2008/11/11 changed by N.Tanaka
// X86_64bit̏ꍇlong8byteƂȂ邽߁CintɕύX
/**
 File Header struct
 \author	Natsu Tanaka
 \date	11 Sep 2006
*/
typedef struct {
	unsigned int ulVersion;	///<\~Japanese o[W
	float fFs;					///<\~Japanese TvOg[Hz]
	unsigned int ulNFFT;		///<\~Japanese FFT[point]
	unsigned int ulWindow;		///<\~Japanese [sample]
	int lNFrame;				///<\~Japanese t[(̏ꍇ͕)
	int lShift;				///<\~Japanese Vtg[sample]
	unsigned int ulNCh;		///<\~Japanese `l
	unsigned int ulLowerFreqLine;	///<\~Japanese gCԍ
	unsigned int ulUpperFreqLine;	///<\~Japanese gCԍ
	unsigned short usDataType;	///<\~Japanese f[^̃^Cv(0:float, 1:complex(float))
	char cPad[22];
} MICARY_SpecFileHeader;

/**
 Spectrumfile information struct
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	//char	cFilename[256]; ///<\~Japanese t@C
	HA_String ha_stFileName; ///<\~Japanese t@C
	FILE *	pfinFile; ///<\~Japanese t@C|C^
	HA_Num	ha_numCurrentFrame; ///<\~Japanese ǂݏoς݃t[[frame]
	MICARY_SpecFileHeader micary_sSpecHeader; ///<\~Japanese XyNgt@Cwb_
} MICARY_SpecFile;

// modified S.Suzuki 2010/03/28
/**
 Wavfile struct for getting a frame data
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
/*
typedef struct{
	MICARY_WavFile	micary_sWavFile; ///<\~Japanese WAVt@C
	HA_Integer	ha_iStartsample; ///<\~Japanese JnԃTv[sample]
	HA_Integer	ha_iEndSample; ///<\~Japanese IԃTv[sample]
} MICARY_SavedGetWavFrame;
*/
// modified S.Suzuki 2010/03/28
/**
 Wavfile struct for setting a frame data
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
/*
typedef struct{
	MICARY_WavFile	micary_sWavFile; ///<\~Japanese WAVt@C
	MICARY_FloatArray	micary_faWave; ///<\~Japanese nf[^
} MICARY_SavedSetWavFrame;
*/
/**
 Spectrumfile struct for getting a frame data
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	MICARY_SpecFile	micary_sSpecFile; ///<\~Japanese XyNgt@C
	HA_Integer	ha_iStartFrame; ///<\~Japanese Jnԃt[[frame]
	HA_Integer	ha_iEndFrame; ///<\~Japanese Iԃt[[frame]
} MICARY_SavedGetSpecFrame;

/**
 Spectrumfile struct for setting a frame data
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	MICARY_SpecFile	micary_sSpecFile; ///<\~Japanese XyNgt@C
	HA_Num	ha_numCurrentFrame; ///<\~Japanese ݍς݃t[[frame]
} MICARY_SavedSetSpecFrame;

/**
 Parameter struct for getting frame data
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	HA_Float	ha_fFs;	///<\~Japanese TvOg[Hz]
	HA_Num	ha_numNFFT; ///<\~Japanese FFT[point]
	HA_Integer	ha_iShift; ///<\~Japanese Vtg[sample](t[̎n[Ot[̏I[)
	HA_Num	ha_numWindow; ///<\~Japanese [sample]
	MICARY_WindowEnum	micary_nwnWindow; ///<\~Japanese ֐
} MICARY_PrmFrame;

/**
 Spectrum struct (Complex)
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	MICARY_ComplexArray	micary_cmpaSpec; ///<\~Japanese XyNg(Complex) 
	HA_Num	ha_numCh; ///<\~Japanese `l
	HA_Integer	ha_iLowerFreq; ///<\~Japanese g[Line]
	HA_Integer	ha_iUpperFreq; ///<\~Japanese g[Line]
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReservedChSize; ///<\~Japanese z̎ۂ̃TCY
	HA_Num	ha_numReservedFreqSize; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_ComplexSpecArray;

/**
 Spectrum struct (Float)
 \author	Natsu Tanaka
 \date 29 Sep 2006
*/
typedef struct{
	MICARY_FloatArray	micary_faSpec; ///<\~Japanese XyNg(Float) 
	HA_Num	ha_numCh; ///<\~Japanese `l
	HA_Integer	ha_iLowerFreq; ///<\~Japanese g[Line]
	HA_Integer	ha_iUpperFreq; ///<\~Japanese g[Line]
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num	ha_numReservedChSize; ///<\~Japanese z̎ۂ̃TCY
	HA_Num	ha_numReservedFreqSize; ///<\~Japanese z̎ۂ̃TCY
#endif
} MICARY_FloatSpecArray;

// ---------------------------------------------------------------------
#endif	// MICARY_struct_h_defined
